@@ -33,11 +33,14 @@ SAB :: <include|exclude> <source1, source2, ... sourceN>
 
 REL :: <include|exclude> <relation1, relation2, ... relationN>
 
+RELA :: <include|exclude> <rela1, rela2, ... relaN>  (optional)
+
 For example, if we wanted to use the MSH vocabulary with only 
 the RB/RN relations, the configuration file would be:
 
 SAB :: include MSH
 REL :: include RB, RN
+RELA :: include inverse_isa, isa
 
 or 
 
@@ -269,10 +272,6 @@ if(defined $opt_socket) {
 
 $umls = UMLS::Interface->new(\%option_hash); 
 die "Unable to create UMLS::Interface object.\n" if(!$umls);
-($errCode, $errString) = $umls->getError();
-die "$errString\n" if($errCode);
-
-&errorCheck($umls);
 
 my @inputarray = ();
 if(defined $opt_infile) {
@@ -301,24 +300,16 @@ foreach my $input (@inputarray) {
 	@c = $umls->getConceptList($input);
     }
     
-    &errorCheck($umls);
-    
     my $printFlag = 0; 
     my $precision = 4;
     my $floatformat = join '', '%', '.', $precision, 'f';
    
     foreach my $cui (@c) {
-	
-	if($umls->validCui($cui)) {
-	    print STDERR "ERROR: The concept ($cui) is not valid.\n";
-	    exit;
-	}
-	
+		
 	#  make certain cui exists in this view
 	if($umls->exists($cui) == 0) { next; }
 	
 	my $paths = $umls->pathsToRoot($cui);
-	&errorCheck($umls);
 	
 	if($#{$paths} < 0) {
 	    print "There are no paths between $term ($cui) and the root.\n";
@@ -340,7 +331,6 @@ foreach my $input (@inputarray) {
 		if( (defined $opt_info) and ($i < $#array) ) {
 		    my $second = $array[$i+1];
 		    my @relations = $umls->getRelationsBetweenCuis($element, $second);
-		    &errorCheck($umls);
 		    print " => @relations => ";
 		}
 	    } print "\n";
@@ -355,15 +345,6 @@ foreach my $input (@inputarray) {
     }
 }
 
-sub errorCheck
-{
-    my $obj = shift;
-    ($errCode, $errString) = $obj->getError();
-    print STDERR "$errString\n" if($errCode);
-    exit if($errCode > 1);
-}
-
-
 ##############################################################################
 #  function to output minimal usage notes
 ##############################################################################
@@ -437,7 +418,7 @@ sub showHelp() {
 #  function to output the version number
 ##############################################################################
 sub showVersion {
-    print '$Id: findPathToRoot.pl,v 1.20 2010/05/11 20:04:46 btmcinnes Exp $';
+    print '$Id: findPathToRoot.pl,v 1.21 2010/05/24 17:57:16 btmcinnes Exp $';
     print "\nCopyright (c) 2008, Ted Pedersen & Bridget McInnes\n";
 }
 
